//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	ColorImage.cc: Implementation of ColorImage methods
//
//	DESCRIPTION:
//		Methods to construct/modify/access ColorImage.
//
//	CHANGE HISTORY:
//		MB, 1999-??-??: 1.0.0: - first version
//		UL, 2000-07-19: 2.0.0: - various changed related to 1.4 overhaul
//							   - added methods to write RGB image as
//								 data cube in primary array
//
//	AUTHORS:
//		M. Beijersbergen (MB)
//		U. Lammers (UL), Mon Jul 17 13:46:09 MET DST 2000
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /usr/local/cvs/sasdev/task_devel/packages/xcolorcod/src/ColorImage.cc,v 1.5 2002/12/12 12:27:37 gvacanti Exp $
//
#include <cmath>

#include "ColorImage.h"
#include "Dal.h"

ColorImage::ColorImage(int nx, int ny) : _pixels(0)
{
	size(nx, ny);
} // ColorImage::ColorImage()


void ColorImage::size(int nx, int ny)
{
	delete _pixels;
	_pixels = new Color[nx*ny];
	_nx = nx;
	_ny = ny;
} // ColorImage::size()


Color& ColorImage::operator()(int ix, int iy)
{
	if (ix < 0 || ix >= _nx ||
		iy < 0 || iy >= _ny)
		return Color::black;

	return _pixels[iy*_nx+ix];
} // ColorImage::operator()


const Color& ColorImage::operator()(int ix, int iy) const
{
	if (ix < 0 || ix >= _nx ||
		iy < 0 || iy >= _ny)
		return Color::black;

	return _pixels[iy*_nx+ix];
} // ColorImage::operator()


double ColorImage::max() const
{
	double cmax = 0;
	for (int iy = 0; iy < _ny; ++iy)
		for (int ix = 0; ix < _nx; ++ix) {
			Color c = (*this)(ix,iy);
			if (c.red > cmax) cmax = c.red;
			if (c.green > cmax) cmax = c.green;
			if (c.blue > cmax) cmax = c.blue;
		}

	return cmax;
} // ColorImage::max()


void ColorImage::normalize(double newmax)
{
	double cmax = max();
	if (cmax <= 0)
		return;

	double factor = newmax/cmax;

	for (int iy = 0; iy < _ny; ++iy)
		for (int ix = 0; ix < _nx; ++ix)
			(*this)(ix,iy) *= factor;
} // ColorImage::normalize()


void ColorImage::log(double decades)
{
	double cmax = max();
	if (cmax <= 0)
		return;

	double logcmax = ::log10(cmax);

	for (int iy = 0; iy < _ny; ++iy)
		for (int ix = 0; ix < _nx; ++ix) {
			Color& c = (*this)(ix,iy);
			if (c.red >= 0) {
				double v = ::log10(c.red) - logcmax + decades;
				c.red = (v >= 0 ? v : 0);
			}
			if (c.green >= 0) {
				double v = ::log10(c.green) - logcmax + decades;
				c.green = (v >= 0 ? v : 0);
			}
			if (c.blue >= 0) {
				double v = ::log10(c.blue) - logcmax + decades;
				c.blue = (v >= 0 ? v : 0);
			}
		}
} // ColorImage::log()


void ColorImage::writePPM(ostream& os) const
{
	os<<"P6"<<endl;
	os<<_nx<<' '<<_ny<<endl;
	os<<max()<<endl;
	unsigned char pixels[_nx*_ny*3];
	unsigned char *ptr = &pixels[0];
	Color *col = _pixels;
	for (int iy = 0; iy < _ny; ++iy)
		for (int ix = 0; ix < _nx; ++ix) {
			*ptr++ = static_cast<unsigned char>(col->red);
			*ptr++ = static_cast<unsigned char>(col->green);
			*ptr++ = static_cast<unsigned char>(col->blue);
			++col;
		}
	os.write(pixels,sizeof(pixels));
} // ColorImage::writePPM()


void ColorImage::writeAsRGBarrays(DataSet *set)
{
	longVec	size;
	size.push_back(_nx);
	size.push_back(_ny);

	//	write red array
	ArrayData *rd = set->addArray("RED", Dal::Int8, size)->data();
	int8 *r = rd->int8Data();
	Color *col = _pixels;
	for (int iy=0; iy<_ny; ++iy)
		for (int ix=0; ix<_nx; ++ix)
			*r++ = static_cast<int8>((col++)->red+.5);

	//	write green array
	ArrayData *gd = set->addArray("GREEN", Dal::Int8, size)->data();
	int8 *g = gd->int8Data();
	col = _pixels;
	for (int iy=0; iy<_ny; ++iy)
		for (int ix=0; ix<_nx; ++ix) {
			*g++ = static_cast<int8>(col->green+.5);
			++col;
	}

	//	write blue array
	ArrayData *bd = set->addArray("BLUE", Dal::Int8, size)->data();
	int8 *b = bd->int8Data();
	col = _pixels;
	for (int iy=0; iy<_ny; ++iy)
		for (int ix=0; ix<_nx; ++ix) {
			*b++ = static_cast<int8>(col->blue+.5);
			++col;
	}

	delete rd;
	delete gd;
	delete bd;
} // ColorImage::writeAsRGBarrays()


void ColorImage::writeAsDataCube(DataSet *set)
{
	longVec	size;
	size.push_back(3);
	size.push_back(_ny);
	size.push_back(_nx);

	normalize(255);
	ArrayData *ad = set->addArray("PRIMARY", Dal::Int8, size)->data();
	int8 *rgb = ad->int8Data();
	longVec posVec(3);
	for (int iy=0; iy<_ny; ++iy)
		for (int ix=0; ix<_nx; ++ix) {
			posVec[2] = ix; posVec[1] = iy;

			posVec[0] = 0; rgb[ad->position(posVec)] =
									static_cast<int8>((*this)(ix, iy).red);
			posVec[0] = 1; rgb[ad->position(posVec)] =
									static_cast<int8>((*this)(ix, iy).green);
			posVec[0] = 2; rgb[ad->position(posVec)] =
									static_cast<int8>((*this)(ix, iy).blue);
		}
	delete ad;
} // ColorImage::writeAsDataCube()


void ColorImage::writeDataSet(const string &name, bool asCube)
{
	DataSet *set = dataSetServer->open(name, DataSet::Create);

	if (asCube)
		writeAsDataCube(set);
	else
		writeAsRGBarrays(set);

	dataSetServer->close(set);
} // ColorImage::writeDataSet()


ColorImage::~ColorImage()
{
	delete[] _pixels;
} // ColorImage::~ColorImage()
