//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	TableBasedPseudoRGBImage : RGB image in the form of a table
//
//	DESCRIPTION:
//		A class that represents an RGB image in the form of a table
//		with three columns "RED"/"GREEN"/"BLUE"
//
//	CHANGE HISTORY:
//		UL, 2000-07-17: 1.0.0: - first version
//		UL, 2000-10-30: 1.1.0: - fixed problem with forgotten data set
//								 close (caused test harness to fail in a weird
//								 manner in lowmem)
//		UL, 2000-11-09: 1.2.0: - No! Above fix only cured the low-mem failure
//								 by pure chance. The problem was really
//								 trying to read a newly created data set
//								 before it is closed:
//									newset = dataSetServer->open(...)
//										populate(newset)
//										read(newset)
//									dataSetServer->close(newset)
//								 Unknown if this is a Dal bug or feature.
//								 New set is now closed before it is being
//								 used
//
//	AUTHOR:
//		U. Lammers (UL), Mon Jul 17 13:46:09 MET DST 2000
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /usr/local/cvs/sasdev/task_devel/packages/xcolorcod/src/TableBasedPseudoRGBImage.cc,v 1.6.2.1 2002/12/12 12:32:33 gvacanti Exp $
//
#include <cstdio>	//	tmpnam()

#include "TableBasedPseudoRGBImage.h"
#include "ConvertableData.h"

#include "metatask.h"
#include "evselect.h"

TableBasedPseudoRGBImage::TableBasedPseudoRGBImage(const string &table,
												   const string &xCol,
												   const string &yCol,
												   const string &eCol,
			 const LinearlyInterpolatedPointsCurve<double,Color> &colorCurve)
			 : _tempTableName("tempcolorset.ds:COLORTAB")
{
	Table *inTab = dataSetServer->open(table, Dal::Read)->table(table);
	unsigned N = inTab->rows();
	const Column *x = inTab->column(xCol),
				 *y = inTab->column(yCol);

	Table *tempTable  = dataSetServer->open(_tempTableName, Dal::Create)
												->addTable(_tempTableName, N);

	tempTable->add(x);
	tempTable->add(y);

	_cdr = tempTable->addColumn("RED", Dal::Real32)->data(),
	_cdg = tempTable->addColumn("GREEN", Dal::Real32)->data(),
	_cdb = tempTable->addColumn("BLUE", Dal::Real32)->data();
	_r = _cdr->real32Data();
	_g = _cdg->real32Data();
	_b = _cdb->real32Data();

	for (unsigned n=0; n<N; ++n)
		_r[n] = _g[n] = _b[n] = 0.f;
	ConvertableData cde(inTab->column(eCol)->data());
	for (unsigned n=0; n<N; ++n) {
		Color col = colorCurve(cde.asDouble(n));
		_r[n] += col.red;
		_g[n] += col.green;
		_b[n] += col.blue;
	}

	delete _cdr;
	delete _cdg;
	delete _cdb;

	dataSetServer->close(tempTable->parent());
	dataSetServer->close(inTab->parent());
} // TableBasedPseudoRGBImage::TableBasedPseudoRGBImage()

//
//	convert to a ColorImage using evselect
//
ColorImage *TableBasedPseudoRGBImage::toColorImage() const
{
	ColorImage *colImage = 0;

	vector<string> colNames(3);
	colNames[0] = "RED";
	colNames[1] = "GREEN";
	colNames[2] = "BLUE";

	ParameterInfo *p = Info::Task::current()->parameters();
	string xcolumn(p->parameter("xcolumn")->stringValue()),
		   ycolumn(p->parameter("ycolumn")->stringValue()),
		   imagebinning(p->parameter("imagebinning")->stringValue());
	double ximagebinsize(p->parameter("ximagebinsize")->realValue()),
		   yimagebinsize(p->parameter("yimagebinsize")->realValue());
	int	   ximagesize(p->parameter("ximagesize")->intValue()),
		   yimagesize(p->parameter("yimagesize")->intValue());
	bool   withxranges(p->parameter("withxranges")->boolValue()),
		   withyranges(p->parameter("withyranges")->boolValue()),
		   squarepixels(p->parameter("squarepixels")->boolValue());
	double ximagemin(p->parameter("ximagemin")->realValue()),
		   yimagemin(p->parameter("yimagemin")->realValue()),
		   ximagemax(p->parameter("ximagemax")->realValue()),
		   yimagemax(p->parameter("yimagemax")->realValue());

	Info::Task *ev = task("evselect");
	ev->setParameter("writedss", false);
	ev->setParameter("updateexposure", false);
	ev->setParameter("withimageset", true);
	ev->setParameter("imagebinning", imagebinning);
	ev->setParameter("squarepixels", squarepixels);
	ev->setParameter("xcolumn", xcolumn);
	ev->setParameter("ximagebinsize", ximagebinsize);
	ev->setParameter("ximagesize", ximagesize);
	ev->setParameter("withxranges", withxranges);
	ev->setParameter("ximagemin", ximagemin);
	ev->setParameter("ximagemax", ximagemax);
	ev->setParameter("ycolumn", ycolumn);
	ev->setParameter("yimagebinsize", yimagebinsize);
	ev->setParameter("yimagesize", yimagesize);
	ev->setParameter("withyranges", withyranges);
	ev->setParameter("yimagemin", yimagemin);
	ev->setParameter("yimagemax", yimagemax);
	ev->setParameter("withimagedatatype", true);
	ev->setParameter("imagedatatype", "Real64");
	for (unsigned n=0; n<colNames.size(); ++n) {
		string imgName("$__tmpimg__$");
		ev->setParameter("imageset", imgName);
		ev->setParameter("withzcolumn", true);
		ev->setParameter("zcolumn", colNames[n]);
		ev->setParameter("table", _tempTableName);

		errstr<<"Constructing "<<colNames[n]<<" image with eveselect ..."
			  <<Sas::message(Msg::App, Msg::Verbose);
		ev->exec();

		Array *arr = dataSetServer->open(imgName, Dal::Read)->array(0);
		ArrayData *ad = arr->data();
		unsigned Nx = ad->size()[0],
				 Ny = ad->size()[1];
		real64 *adata = ad->real64Data();
		if (!colImage)
			colImage = new ColorImage(Nx, Ny);
		longVec posVec(2);
		for (unsigned nx=0; nx<Nx; ++nx)
			for (unsigned ny=0; ny<Ny; ++ny) {
				posVec[0] = nx; posVec[1] = ny;
				(*colImage)(nx, ny).component(n) = adata[ad->position(posVec)];
			}
		delete ad;

		dataSetServer->close(arr->parent());
		dataSetServer->remove(imgName);
	}

	return colImage;
} // TableBasedPseudoRGBImage::toColorImage()


TableBasedPseudoRGBImage::~TableBasedPseudoRGBImage()
{
	dataSetServer->remove(_tempTableName);
} // TableBasedPseudoRGBImage::~TableBasedPseudoRGBImage()
